@@ -43,6 +43,7 @@ from sklearn.metrics import precision_recall_fscore_support
 from sklearn.metrics import accuracy_score, auc, average_precision_score, confusion_matrix, roc_curve, precision_recall_curve
 from numpy import interp
 from sklearn.metrics import  roc_auc_score, average_precision_score 
+from sklearn.model_selection import  StratifiedShuffleSplit
 #import src.Prj_Core.core_helper.model.neg_bagging_fraction__lgb_model as nbf_lgb_model
 #import src.Prj_Core.core_helper.model.scale_pos_weight__lgb_model as spw_lgb_model
 #import src.Prj_Core.core_helper.model.custom_bagging__lgb_model as cb_lgb_model
@@ -51,7 +52,10 @@ from sklearn.metrics import  roc_auc_score, average_precision_score
 #import model.scale_pos_weight__lgb_model as spw_lgb_model
 #import model.custom_bagging__lgb_model as cb_lgb_model
 
-def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=None,y_test=None,params=None,max_iter_cls=None, cls_ini_opt=[] ,enable_cls_opt = False, enable_umbral_opt=False,metric='average_precision',num_rounds=10000,early_stop=400,log=0,api="train_api",sesgo=0.15,random_state=42,url=None, cv=False,cv_n_split=None, print_consola=True):
+def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=None,y_test=None,params=None,max_iter_cls=None,
+                                  cls_ini_opt=[] ,enable_cls_opt = False, enable_umbral_opt=False,metric='average_precision',
+                                  num_rounds=10000,early_stop=400,log=0,api="train_api",sesgo=0.15,random_state=42,url=None, 
+                                  cv=False,cv_n_split=None, print_consola=True,   shuffle_cv=False,  shuffle_cv_test_size=None):
     start = time.time()
     
     if url is not None:
@@ -146,7 +150,7 @@ def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=
     else:
         num_rounds_cv = model.best_iteration()
         
-    
+  
     if cv : 
         
         if url is None:
@@ -156,7 +160,8 @@ def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=
         
             args_cv = { 'X_train':X_train, 'y_train': y_train, 'X_test': X_test, 'y_test': y_test, 'params':params,
                         "metric":metric,"num_rounds":num_rounds_cv,"random_state":random_state,
-                        "api":api , "url":url , "cv_n_split" :cv_n_split
+                        "api":api , "url":url , "cv_n_split" :cv_n_split, "shuffle_cv" :shuffle_cv,
+                        "shuffle_cv_test_size" :shuffle_cv_test_size,
                         }      
             
             generate_cv_result(**args_cv)    
@@ -182,19 +187,25 @@ def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=
 
 
 def generate_cv_result(X_train=None,y_train=None,X_test=None,y_test=None,api=None, num_rounds = None , metric=None, 
-                       params =None ,  url=None, cv_n_split=None, random_state=42):
+                       params =None ,  url=None, shuffle_cv = False ,shuffle_cv_test_size=None, cv_n_split=None, random_state=42):
     
    print(X_train.shape)
    X_t_ = X_train.append(X_test)
    y_ = y_train.append(y_test)
    
-   if cv_n_split is None:
-       test_size = get_test_size(X_t_)
-       n_s = round(1/test_size)
+   if shuffle_cv:
+       folds = StratifiedShuffleSplit(n_splits=cv_n_split, test_size=shuffle_cv_test_size, random_state=random_state)
+       print("StratifiedShuffleSplit")
    else:
-       n_s = cv_n_split
-
-   folds = StratifiedKFold(n_splits=n_s, shuffle=True, random_state=random_state) 
+   
+       if cv_n_split is None:
+           test_size = get_test_size(X_t_)
+           n_s = round(1/test_size)
+       else:
+           n_s = cv_n_split      
+         
+       print("StratifiedKFold")
+       folds = StratifiedKFold(n_splits=n_s, shuffle=True, random_state=random_state) 
    print("##########################################")
    print("params: : ", params)
    print("num_rounds_cv : ",num_rounds)
@@ -232,7 +243,7 @@ def generate_cv_result(X_train=None,y_train=None,X_test=None,y_test=None,api=Non
        
              
 
-def kpis_cross_validation():
+def kpis_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):
     
     list_train_precision = []
     list_train_recall = []
@@ -247,12 +258,16 @@ def kpis_cross_validation():
     list_valid_average_precision = []
     list_valid_roc_auc = []
     
-    path_cv = "CV"
+
     
-    for i in range(5):
+    for i in range(n_splits):
         
-    
-       df_cv_fold = pd.read_csv(f"{path_cv}/y_train_cv_{i}.csv", index_col=False)
+       df_cv_fold = pd.DataFrame()           
+       for path_ in list_path:
+           print(" path_ :  ", path_)   
+           df_cv_fold_ = pd.read_csv(f"{path_}/y_train_cv_{i}.csv", index_col=False)
+           df_cv_fold = df_cv_fold.append(df_cv_fold_)
+           
        y_train_cv = df_cv_fold.y_train_cv
        y_prob_uno_train_cv = df_cv_fold.y_prob_uno_train_cv       
        y_pred_train_cv = np.round(y_prob_uno_train_cv, 0)
@@ -268,7 +283,12 @@ def kpis_cross_validation():
        list_train_roc_auc.append(roc_auc_train)
        
        
-       df_cv_fold = pd.read_csv(f"{path_cv}/y_test_cv_{i}.csv", index_col=False)
+       df_cv_fold = pd.DataFrame()           
+       for path_ in list_path:
+           print(" path_ :  ", path_)   
+           df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
+           df_cv_fold = df_cv_fold.append(df_cv_fold_)      
+        
        y_test_cv = df_cv_fold.y_test_cv
        y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv       
        y_pred_valid_cv = np.round(y_prob_uno_test_cv, 0)
@@ -285,59 +305,83 @@ def kpis_cross_validation():
        
    
 
+    mean_valid_precision = statistics.mean(list_valid_precision)           
+    std_valid_precision = statistics.pstdev(list_valid_precision)
+    min_valid_precision = min(list_valid_precision)           
+    max_valid_precision = max(list_valid_precision)      
     mean_train_precision = statistics.mean(list_train_precision)           
     std_train_precision = statistics.pstdev(list_train_precision)
-    mean_train_recall = statistics.mean(list_train_recall)           
-    std_train_recall = statistics.pstdev(list_train_recall)
-    mean_train_f1 = statistics.mean(list_train_f1)           
-    std_train_f1 = statistics.pstdev(list_train_f1)    
-    mean_train_average_precision = statistics.mean(list_train_average_precision) 
-    std_train_average_precision = statistics.pstdev(list_train_average_precision) 
-    mean_train_roc_auc = statistics.mean(list_train_roc_auc)           
-    std_train_roc_auc = statistics.pstdev(list_train_roc_auc)
-             
-    mean_valid_precision = statistics.mean(list_valid_precision)           
-    std_valid_precision = statistics.pstdev(list_valid_precision) 
+    min_train_precision = min(list_train_precision)           
+    max_train_precision = max(list_train_precision)  
+    
+    
     mean_valid_recall = statistics.mean(list_valid_recall)           
     std_valid_recall = statistics.pstdev(list_valid_recall)
+    min_valid_recall = min(list_valid_recall)           
+    max_valid_recall = max(list_valid_recall)    
+    mean_train_recall = statistics.mean(list_train_recall)           
+    std_train_recall = statistics.pstdev(list_train_recall)
+    min_train_recall = min(list_train_recall)           
+    max_train_recall = max(list_train_recall) 
+    
+    
     mean_valid_f1 = statistics.mean(list_valid_f1)           
     std_valid_f1 = statistics.pstdev(list_valid_f1)
+    min_valid_f1 = min(list_valid_f1)           
+    max_valid_f1 = max(list_valid_f1)       
+    mean_train_f1 = statistics.mean(list_train_f1)           
+    std_train_f1 = statistics.pstdev(list_train_f1)   
+    min_train_f1 = min(list_train_f1)           
+    max_train_f1 = max(list_train_f1)   
+    
+
     mean_valid_average_precision = statistics.mean(list_valid_average_precision) 
     std_valid_average_precision = statistics.pstdev(list_valid_average_precision)
+    min_valid_average_precision = min(list_valid_average_precision)           
+    max_valid_average_precision = max(list_valid_average_precision)     
+    mean_train_average_precision = statistics.mean(list_train_average_precision) 
+    std_train_average_precision = statistics.pstdev(list_train_average_precision) 
+    min_train_average_precision = min(list_train_average_precision)           
+    max_train_average_precision = max(list_train_average_precision)     
+    
+    
     mean_valid_roc_auc = statistics.mean(list_valid_roc_auc)           
     std_valid_roc_auc = statistics.pstdev(list_valid_roc_auc)
-   
-    result_kpis = {   
-        
-        'mean_train_precision':mean_train_precision,
-        'std_train_precision':std_train_precision,
-        'mean_train_recall':mean_train_recall,
-        'std_train_recall':std_train_recall, 
-        'mean_train_f1':mean_train_f1,
-        'std_train_f1':std_train_f1 ,
-        'mean_train_average_precision':mean_train_average_precision,
-        'std_train_average_precision':std_train_average_precision,       
-        'mean_train_roc_auc':mean_train_roc_auc,
-        'std_train_roc_auc':std_train_roc_auc   ,               
+    min_valid_roc_auc = min(list_valid_roc_auc)           
+    max_valid_roc_auc = max(list_valid_roc_auc)    
+    mean_train_roc_auc = statistics.mean(list_train_roc_auc)           
+    std_train_roc_auc = statistics.pstdev(list_train_roc_auc)
+    min_train_roc_auc = min(list_train_roc_auc)           
+    max_train_roc_auc = max(list_train_roc_auc)  
 
-        'mean_valid_precision':mean_valid_precision,
-        'std_valid_precision':std_valid_precision,
-        'mean_valid_recall':mean_valid_recall,
-        'std_valid_recall':std_valid_recall, 
-        'mean_valid_f1':mean_valid_f1,
-        'std_valid_f1':std_valid_f1 ,
-        'mean_valid_average_precision':mean_valid_average_precision,
-        'std_valid_average_precision':std_valid_average_precision,       
-        'mean_valid_roc_auc':mean_valid_roc_auc,
-        'std_valid_roc_auc':std_valid_roc_auc   ,        
     
-                
-    }  
-     
-    return result_kpis
+    result_kpis = [
+                   {"Metric":"Precision","Splits":n_splits,"Data":"Test","Mean":mean_valid_precision,"SD":std_valid_precision,"Min":min_valid_precision,"Max":max_valid_precision},
+                   {"Metric":"Precision","Splits":n_splits,"Data":"Train","Mean":mean_train_precision,"SD":std_train_precision,"Min":min_train_precision,"Max":max_train_precision},
+                   
+                   {"Metric":"Recall","Splits":n_splits,"Data":"Test","Mean":mean_valid_recall,"SD":std_valid_recall,"Min":min_valid_recall,"Max":max_valid_recall},
+                   {"Metric":"Recall","Splits":n_splits,"Data":"Train","Mean":mean_train_recall,"SD":std_train_recall,"Min":min_train_recall,"Max":max_train_recall},
+    
+                   {"Metric":"F1","Splits":n_splits,"Data":"Test","Mean":mean_valid_f1,"SD":std_valid_f1,"Min":min_valid_f1,"Max":max_valid_f1},
+                   {"Metric":"F1","Splits":n_splits,"Data":"Train","Mean":mean_train_f1,"SD":std_train_f1,"Min":min_train_f1,"Max":max_train_f1},
+    
+                   {"Metric":"Average Precision","Splits":n_splits,"Data":"Test","Mean":mean_valid_average_precision,"SD":std_valid_average_precision,"Min":min_valid_average_precision,"Max":max_valid_average_precision},
+                   {"Metric":"Average Precision","Splits":n_splits,"Data":"Train","Mean":mean_train_average_precision,"SD":std_train_average_precision,"Min":min_train_average_precision,"Max":max_train_average_precision},
+    
+                   {"Metric":"ROC AUC","Splits":n_splits,"Data":"Test","Mean":mean_valid_roc_auc,"SD":std_valid_roc_auc,"Min":min_valid_roc_auc,"Max":max_valid_roc_auc},
+                   {"Metric":"ROC AUC","Splits":n_splits,"Data":"Train","Mean":mean_train_roc_auc,"SD":std_train_roc_auc,"Min":min_train_roc_auc,"Max":max_train_roc_auc},               
+                   ]
+    
+    df = pd.DataFrame(result_kpis)
+    
+    if url_out is None:
+        return df
+    else:
+        df.to_excel(url_out+f"kpis_cross_validation_{postfix}.xlsx")  
+ 
+ 
 
-
-def roc_curva_cross_validation(list_path=[], n_splits=5):
+def roc_curva_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):
     
     tprs = []
     aucs = []
@@ -345,9 +389,11 @@ def roc_curva_cross_validation(list_path=[], n_splits=5):
 
     #list_path = ["CV","CV2"]
     
-    for i in range(n_splits):
-    
-       df_cv_fold = pd.DataFrame()
+       
+    for i in range(n_splits):   
+        
+       df_cv_fold = pd.DataFrame()   
+        
        for path_ in list_path:
            print(" path_ :  ", path_)
            df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
@@ -364,8 +410,8 @@ def roc_curva_cross_validation(list_path=[], n_splits=5):
        #plt.plot(fpr, tpr, lw=1, alpha=0.3, label='ROC fold %d (AUC = %0.2f)' % (i, roc_auc))
        i += 1  
     
-    
-    plt.plot([0, 1], [0, 1], linestyle='--', lw=2, color='r', label='Luck', alpha=.8)  
+    print("df_cv_fold.shape: ", df_cv_fold.shape)
+    plt.plot([0, 1], [0, 1], linestyle='--', lw=1, color='r', label='Reference', alpha=.8)  
    
     mean_tpr = np.mean(tprs, axis=0)
     mean_tpr[-1] = 1.0
@@ -376,15 +422,22 @@ def roc_curva_cross_validation(list_path=[], n_splits=5):
     std_tpr = np.std(tprs, axis=0)
     tprs_upper = np.minimum(mean_tpr + std_tpr, 1)
     tprs_lower = np.maximum(mean_tpr - std_tpr, 0)
-    plt.fill_between(mean_fpr, tprs_lower, tprs_upper, color='grey', alpha=.5, label=r'$\pm$ 1 std. dev.')
+    plt.fill_between(mean_fpr, tprs_lower, tprs_upper, color='grey', alpha=.5, label=r' std. dev.')
 
     plt.xlim([-0.05, 1.05])
     plt.ylim([-0.05, 1.05])
     plt.xlabel('False Positive Rate')
-    plt.ylabel('True Positive Rate')
-    plt.title("Curva ROC")
+    plt.ylabel('True Positive Rate')  
+    plt.title(f"Curva ROC (n_splits:{n_splits})")
     plt.legend(loc="lower right")
-    plt.show()
+    #plt.show()
+    
+    if url_out is None:
+        plt.show()
+    else:
+        hg.validar_directorio(url_out)  
+        plt.savefig(url_out+f'roc_curva_cv_{postfix}.png', bbox_inches='tight')
+        plt.close()
 
 
 def modelar_clasificacion_binaria_rscv(strategy, X_train,y_train=None,X_test=None,y_test=None,score_rs=None,params=None,param_dist=None, n_iter=None,n_jobs=None,url=None,print_consola=True):
